<?php

declare(strict_types=1);

namespace App\Services\QueryRequestModifiers\Post;

use App\Http\Requests\Post\ListPostRequest;
use App\Models\Post;
use App\Services\QueryRequestModifiers\Filterable;
use Illuminate\Database\Eloquent\Builder;

class PostFilter
{
    /**
     * @use Filterable<Post, PostFilterDTO>
     */
    use Filterable;

    /**
     * @param Builder<Post> $query
     * @return Builder<Post>
     */
    public function apply(Builder $query, ?PostFilterDTO $filters): Builder
    {
        return $this->applyFilterable($query, $filters);
    }

    public function makeFromRequest(ListPostRequest $request): ?PostFilterDTO
    {
        return $this->makeFilterableFromRequest($request, PostFilterDTO::class);
    }

    /**
     * @return array<string, string>
     */
    public function validateRules(): array
    {
        return [
            'title' => 'sometimes|string',
            'content' => 'sometimes|string',
        ];
    }


    /**
     * @return array<int, string>
     */
    protected static function keys(): array
    {
        return ['title', 'content'];
    }
}
